/* struct2.h: 

This is the structure header file. It contains all the structures
that will be used throughout the game. It is essential to understand
the structures to understand the game.

*/

/***********************************/
/* range of alignments enumeration */
/***********************************/
typedef enum align
{
	evil, horrible, bad, naughty, nuetral, nice, good, wonderful, saintly
} align;

/***********************************/
/* range of locations in the map   */
/* volumes                         */
/***********************************/
typedef enum locations
{
	low, middle, high
}               locations;

/***********************************/
/* range of objects enumeration    */
/***********************************/
typedef enum object_type
{
	null,														/* for object links                   */
	player,                         /* special type for players           */
	animate,                        /* monsters and things that move      */
	inanimate,                      /* that which cannot move on its own  */
	wall,                           /* walls...                           */
	floor,                          /* floors....                         */
	switch,                         /* operators (buttons,levers.....)    */
	door,                           /* doors...                           */
	teleport,                       /* teleporters                        */
	NUM_TYPES                       /* dummy variable for counting        */
}object_type;

/***********************************/
/* statistics for players and for  */
/* animate objects                 */
/***********************************/
typedef struct statsstruct
{
	align           alignment;

	unsigned int    exper,	       /* experience points                     */
	                level,	       /* what level you are                    */
	                str,	         /* strength 1 unit = lift 10 pounds      */
	                sma,	         /* intelligence 1 unit = 10 iq           */
	                wis,	         /* wisdom 1 unit = ?                     */
	                dex,	         /* dexterity 1 unit = ?                  */
	                con,	         /* constitution 1 unit = ?               */
	                luck,	         /* luck 1 unit = bonuses                 */
	                looks,	       /* just how ugly you are 20   immaculate */
	                cha,	         /* your personality 20 = ?               */
	                speed;	       /* affects update speed                  */

	int             food,	         /* food in your stomach                  */
	                water,	       /* water in your body                    */
	                damage;	       /* how much damage you do                */
}               statsstruct;

/*********************************/
/* physical and mental attribute */
/* structure data                */
/*********************************/
typedef struct attrstruct
{
	short int       phys,
	                physmax,       /* physical points                       */
	                mental,
	                mentmax;       /* mental points for spells              */
	int             weight,
                  bulk,
                  maxbulk;

}attrstruct;

/*********************************/
/* for the various bitmaps that  */
/* have to be place for an object*/
/*********************************/
typedef struct pictstruct
{
	unsigned short  picture[4];
	bitmap         *next;	       /* animation???? */
}pictstrutct;

/*********************************/
/* for objects with messages     */
/*********************************/
typedef struct messagestruct
{
	char           *text;
	message        *next;
} messagestruct;

/*********************************/
/* status bits for objects       */
/*********************************/
typedef struct obbits{
	unsigned int phased          : 1;   /* pass thru walls?       */
	unsigned int blocks_view     : 1;   /* can see past?          */
	unsigned int blocks_movement : 1;   /* can move past          */
	unsigned int is_removed      : 1;   /* is not in a map?       */
	unsigned int is_freed        : 1;   /* has been freed?        */
	unsigned int is_pl_visible   : 1;   /* is visible to players? */
	unsigned int invisible       : 1;   /* invisible object?      */
	/* many more to come....*/
} statbit;

/*********************************/
/*status bits for players        */
/*********************************/
typdef struct plbits {
	unsigned int dm_power        : 1;  /* can perform dm actions?    */
	unsigned int text_changed    : 1;  /* has text window changed?   */
	unsigned int stats_changed   : 1;  /* has stats window changed?  */
	unsigned int view_changed    : 1;  /* has view window changed?   */
	unsigned int items_chaged    : 1;  /* has items window changed?  */
	/* many more to come... */
} plbits;

/*********************************/
/* composition of an object      */
/*********************************/
typedef struct compstruct {
	unsigned int wood            : 1;
	unsigned int hardwood        : 1;
	unsigned int plastic         : 1;
	unsigned int rubber          : 1;
	unsigned int glass           : 1;
	unsigned int paper           : 1;
	unsigned int mineral         : 1;
	unsigned int metal           : 1;
	} compstruct;

/*********************************/
/* resistances of an object      */
/*********************************/
typedef struct resiststruct {
	int fire            : 4;
	int cold            : 4;
	int electric        : 4;
	int physical        : 4;
	int mental          : 4;
	int poison          : 4;
	} resiststruct;
	    
/*********************************/
/* the actual object structure!  */
/* a union of the above structs  */
/* based on type of object       */
/*********************************/  
typedef struct object {
  object_type 					type;
  unsigned char         subtype;
  
  bitmap 							 	picnum;    
  messagestruct				 	*message;  
    
  unsigned int          x, y, z,
                        map,
                                               
  unsigned char         facing;
  
  object *above,
  			 *below,
  			 *next,
  			 *link;
  
  switch (type) {
  
    /*rahul.... perhaps we should UNION? how can i be sure that the    */
    /*pointers are in the right place all the time? This is ESSENTIAL! */
    case null:
      object *head,*link;
      break;
 		
 		/* this needs help.....*/
    case exit:
      char name [NAMELENGTH + 1];
      unsigned int  xto,yto,
                    mapto,
                    cost_type,
                    cost;
      break;

    case player:
      char name [NAMELENGTH + 1];
      playerstruct *pldata;
      object  *right,*left,*torso,*legs,*head,*finger,*neck,
              *holding,
              *owner;
      statsstruct stats;      
      break;

    case animate:
      char name [NAMELENGTH + 1];
      queststruct *quest
      object  *right,*left,*torso,*legs,*head,*finger,*neck,
              *holding,
              *owner;
      statsstruct stats;
      break;
      
    case inanimate:
  		char name [NAMELENGTH +1];
  		object *holding,
  					 *owner;
  		break;
    case floor:
  		
    case wall:
  
    case door:
    
    case switch:
    
    case spell:
    
    };      
  } object;
  
} object;


struct queststruct {
       int          assoc_quest,   /* what quest this object is */
                                   /* associated with           */
                    action_mask;   /* what action will set the  */
                                   /* bits in the playerstruct  */
  } queststruct;

struct drawdata {
       Drawable   window;
       GC         itembg_gc,itemfg_gc,itemxor_gc,text_gc;
       XSizeHints *size_hint;
       } windowdata;

typedef struct playerstruct {
  int   questcomp,            /* quest completed bits                 */
    queststart,               /* quest started bits                   */
    questrewa;                /* recieved reward for quest bits       */

  struct playerstruct *next,  /* if in a group this points to next    */
                              /* player in group                      */
                      *leader;/* leader of group                      */
  char     *display_name;                          
  Display  *display;            /* what display the player is on        */
  Window   root;                /* display root                         */
  Pixmap   viewpic;
  unsigned long blackpix,whitepix;
  short int tx,ty,th,tc,tw,tm,ts,sc;
  struct   drawdata   main,     /*window stuff as defined above */
                      view,
                      text,
                      item,
                      stats,
                      controls,
                      viewpix,
                      itempix;
  XFontStruct   *itemfont,*cursors,*textfont;
  char info [80][80],
       parsing [255],
       previous[255];
  struct object *pets;      /* pointer to player pets          */
  struct object *object;    /* the object which is the player  */
  }playerstruct;

typedef struct stck{
               struct   object *background,
                               *items,
                               *moving,
                               *top,
                               *bottom;
       } stck;
struct squarestruct { 
       struct   stck vol[high];   /* the volume of pointers to  */
                                  /* objects [low] = lowest*/
    } squarestruct;  

struct map {
  struct squarestruct tiles [20][20];   /* maps are not dynamic....*/
                                        /* for now!                */
  int level;
  struct map *link;
  }map;
  
      
  
