/*********************************************************/
/* file: Input.c                                         */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 april 93                                 */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  None as yet. if you add place your name here or below*/
/*                                                       */
/* description:                                          */
/*  input.c contains routins to handle all types of      */
/* input from a player, many routines to retrieve and    */
/* recieve different types of data are provided for use  */
/* in object handlers                                    */
/*                                                       */
/* LICENSING:                                            */
/*   a full license  is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, and is  named LICENSE     */
/*********************************************************/
#include <standard.h>

/*this should actually use a tree structure*/
void
insert_key(playerstruct *pl,KeySym key,int function)
{
  keystruct *tmp = (keystruct *) malloc (sizeof(keystruct));
  
  tmp->keysym = key;
  tmp->function = function;

  tmp->next = pl->keys;
  
  pl->keys = tmp;
 }

void set_key (playerstruct *pl,char *res_name,int function,char *defaultkey)
{
  char *tmp;
  /*
  if (NO_RESOURCES(pl))
    fprintf (stderr,"no resources for key %s",res_name);
  else
    fprintf (stderr,"resource located for key %s",res_name);*/
  if (NO_RESOURCES(pl) ||
  (tmp = XGetDefault(pl->display,RES_NAME,res_name))==NULL)
  {
    /*fprintf (stderr,"\tinserting default %s as xk %s\n",
    defaultkey,XKeysymToString(XStringToKeysym(defaultkey)));*/
    insert_key(pl,XStringToKeysym (defaultkey),function);
  }
  else
  {
    /*fprintf (stderr,"\tinserting reskey %s as xk %s \n",
    tmp,XKeysymToString(XStringToKeysym(tmp))); */
    insert_key(pl,XStringToKeysym (tmp),function);
  }
}

void
get_player_keys (playerstruct *pl)
{
  char *tmp;
  
  tmp = XGetDefault (pl->display,RES_NAME,"resource_keys");
  
  if (tmp == NULL || (strcmp(tmp,"true") && strcmp(tmp,"True")))
    SET_NO_RESOURCES(pl);
  
  set_key (pl,"move_f_key",move_f,"t");
  set_key (pl,"move_fr_key",move_fr,"y");
  set_key (pl,"move_fl_key",move_fl,"r");
  set_key (pl,"move_l_key",move_l,"f");
  set_key (pl,"move_r_key",move_r,"h");
  set_key (pl,"move_b_key",move_b,"b");
  set_key (pl,"move_bl_key",move_bl,"v");
  set_key (pl,"move_br_key",move_br,"n");
  set_key (pl,"apply_f_key",apply_f,"T");
  set_key (pl,"apply_fr_key",apply_fr,"Y");
  set_key (pl,"apply_fl_key",apply_fl,"R");
  set_key (pl,"apply_l_key",apply_l,"L");
  set_key (pl,"apply_r_key",apply_r,"H");
  set_key (pl,"apply_b_key",apply_b,"B");
  set_key (pl,"apply_bl_key",apply_bl,"V");
  set_key (pl,"apply_br_key",apply_br,"N");
  set_key (pl,"turn_right_key",turn_right,"j");
  set_key (pl,"turn_left_key",turn_left,"d");
  set_key (pl,"pickup_key",pickup,"comma");
  set_key (pl,"drop_key",drop,"slash");
  set_key (pl,"interact_mode_key",interact_mode,"i");
  set_key (pl,"interact_key",interact_with,"I");
  set_key (pl,"use_mode_key",use_mode,"u");
  set_key (pl,"use_key",use_obj,"U");
  set_key (pl,"container_key",container,"c");
  set_key (pl,"query_key",query,"questiondown");
  set_key (pl,"equip_key",equip,"e");
  set_key (pl,"repeat_key",repeat,"period");
  set_key (pl,"parser_key",parser,"apostrophe");
  set_key (pl,"aggressive_key",aggressive,"a");
  set_key (pl,"shift_left_key",shift_left,"k");
  set_key (pl,"shift_right_key",shift_right,"l");
  set_key (pl,"shift_up_key",shift_up,"less");
  set_key (pl,"shift_down_key",shift_down,"greater");
  /*set_key (pl,"",move_fl,"g");*/
}

    
void
apply_bstate(playerstruct * pl, int dir)
{
  object         *tobj;
  if (!OB_DIR_IN_MAP(pl->obj, (dir + pl->obj->facing) % 8))
    return;
  if ((dir < 2) || (dir > 6))
  {
    switch (pl->buttonstate)
    {
    case (1):

      tobj = PL_ADJ_OBJ(pl, dir);
      SENDRQ(tobj, interact) (tobj, pl->obj);
      break;

    case (2):
      if (!PL_HAS_USED(pl))
      {
        tobj = AN_OB_ITEM(pl->obj).held;
        if (tobj != NULL)
        {
          SENDRQ(tobj, use) (tobj, pl->obj, PL_ADJ_OBJ(pl, dir), dir);
        } else
        {
          attack_object(PL_ADJ_OBJ(pl, dir), pl->obj, NULL);
        }
        SET_PL_HAS_USED(pl);
      }
      break;
    }
  } else
    switch (dir)
    {
    case (0):
    case (1):
    case (7):
      break;
    case (2):
    case (3):
      pl->obj->facing = (pl->obj->facing + 2) % 8;
      break;
    case (4):
      if (!PL_HAS_MOVED(pl))
      {
        move_object(pl->obj, (pl->obj->facing + 4) % 8);
        SET_PL_HAS_MOVED(pl);
      }
      break;
    case (5):
    case (6):
      pl->obj->facing = (pl->obj->facing + 6) % 8;
      break;
    default:
      fprintf(stderr, "direction range error in character");
      break;
    }
}

/******************************************/
/* reset_normal_controllers               */
/*    reset_normal_controllers will set   */
/* the players control routines and       */
/* objects back to the default/normal     */
/* values. Objects that grab control      */
/* should use this routine. (must?)       */
/******************************************/
void
reset_normal_controllers(playerstruct * pl)
{
  /* reset the control objects */
  pl->key_object = pl->button_object = pl->obj;

  /* reset the handlers */
  pl->key_control = (&get_single_key);
  pl->button_control = (&get_simple_buttonpress);
  pl->parsing[0] = '\0';
}


/*******************************************/
/* parse_command                           */
/*    parse_command will essentially pass  */
/* control to the appropriate command      */
/* driver. It does so by hashing the       */
/* command name on the input line and then */
/* searching the hash table for the command*/
/* entry which matches                     */
/*******************************************/
void
parse_command(playerstruct * pl)
{
  int             hashlock,
                  rehash;
  char            parsetemp[16];
  command        *ctemp;

  /* attempt parse only if player typed something */
  if (pl->parsing != '\0')
  {
    pl->key_object = pl->obj;
    sscanf(pl->parsing, "%16s", parsetemp);

    /* get hash and rehash values */
    hashlock = name_hash(parsetemp,COMMAND_HASH_SIZE);
    rehash = name_rehash(parsetemp,COMMAND_HASH_SIZE);

    /* search for matching rehash in the specified hash location */
    for (ctemp = commlist[hashlock]; (ctemp != NULL) && (ctemp->rehash != rehash);
         ctemp = ctemp->next);

    /* execute command only if found a match in table */
    /* and it is legal for this player to execute    */
    if ((ctemp == NULL) || (IS_DM_COMMAND(ctemp) && !PL_HAS_DM_POWER(pl)))
      write_text(pl, "no such command...");
    else
      DO_COMM_CALLBACK(ctemp, pl);
  }
}

/**********************************/
/* get_single_key                 */
/*    get_single_key will take a  */
/* single buttonpress event and   */
/* pass the data directly to the  */
/* current key object via the     */
/* recieve key request            */
/**********************************/
void
get_single_key(playerstruct * pl, XEvent event)
{
  char            tmpbuf[128];
  KeySym          key;
  XComposeStatus  compose;
  int             k;

  k = XLookupString(&event.xkey, tmpbuf, 10, &key, &compose);
  /* fprintf (stderr,"%s\n",XKeysymToString(key)); */
  SENDRQ(pl->key_object, recieve_keyboard) (pl->key_object, pl->obj, &key);
}

void
get_line(playerstruct * pl, XEvent event)
{
  char            tmpbuf[128];

  KeySym          key;
  XComposeStatus  compose;

  XLookupString(&event.xkey, tmpbuf, 10, &key, &compose);
  if (isascii(key))
  {
    if (pl->px < 255)
    {
      pl->parsing[pl->px++] = (char) key;
      write_parsing(pl);
    }
  } else
    switch ((int) key)
    {
    case (XK_Return):
      pl->parsing[pl->px] = '\0';
      write_text(pl, pl->parsing);
      clear_parsing(pl);
      SENDRQ(pl->key_object, recieve_keyboard) (pl->key_object, pl->parsing);
      pl->px = 0;
      break;
    case (XK_Delete):
    case (XK_BackSpace):
    case (XK_Left):
      if (pl->px > 0)
      {
        pl->parsing[--pl->px] = '\0';
        clear_parsing(pl);
        write_parsing(pl);
      }
      break;
    default:
      break;
    }
}

void
decode_keycommand(playerstruct * pl, KeySym * key_point)
{
  char            tmpbuf[100];
  KeySym          key = *key_point;
  
  keystruct       *curr;
  object *tmp;
  
  /* we can get this down to log time with tree */
  /* and possibly store the callback in the tree*/
  /* itself yeilding log time compare and exec  */
  /* instead of linear, the way it is now       */
  for (curr = pl->keys;curr != NULL && curr->keysym != key;curr= curr->next);
  if (curr == NULL)
  {
    return;
  }
  
    switch ((int) curr->function)
    {
    case (repeat):
      write_text(pl, pl->parsing);
      parse_command(pl);
      break;
    case (query):
      sprintf(tmpbuf, "X:%d  Y:%d  Z:%d", pl->obj->x, pl->obj->y, pl->obj->z);
      write_text(pl, tmpbuf);
      sprintf(tmpbuf, "facing: %s", dir_names[pl->obj->facing]);
      write_text(pl, tmpbuf);
    /*  testdraw(pl);*/
      break;
    case (shift_left):
      shift_items_left(pl);
      break;
    case (shift_right):
      shift_items_right(pl);
      break;
    case (shift_up):
      shift_items_up(pl);
      break;
    case (shift_down):
      shift_items_down(pl);
      break;
    case (use_obj):
      SENDRQ(pl->top_held,use)(pl->top_held,pl->obj,
      MAP_ADJ_OBJ(pl->obj,pl->obj->facing),pl->obj->facing);
      break;
    case (interact_with):
      SENDRQ(pl->top_held,interact)(pl->top_held,pl->obj);
      break;
      case (pickup):
      if (pl->itemmode == 1)
      {
        if (pl->top_held == pl->obj->holding)
        {
          pickup_object(pl->obj, HEAD(pl->obj->below));
          pl->top_held = pl->obj->holding;
        } else
        {
          pickup_object(pl->obj, HEAD(pl->obj->below));
        }
      } else
        {
        tmp = pl->top_held->below;
        pickup_object(pl->obj, HEAD(pl->top_held));
        pl->top_held = tmp;
        }
      SET_PL_ITEM_CHANGED(pl);
      break;
    case (drop):
      if (pl->top_held == NULL)
        write_text(pl,"Nothing to drop");
      else
      if (IS_EQUIPPED(pl->top_held))
        write_text(pl, "Unequip it Before attempting to Drop");
      else
        drop_object(pl->obj, pl->top_held);
      SET_PL_ITEM_CHANGED(pl);
      break;
    case (interact_mode):
      pl->buttonstate = 1;
      SET_PL_COMS_CHANGED(pl);
      break;
    case (use_mode):
      pl->buttonstate = 2;
      SET_PL_COMS_CHANGED(pl);
      break;
    case (equip):
      equip_object(pl->obj, pl->top_held);
      SET_PL_STATS_CHANGED(pl);
      SET_PL_ITEM_CHANGED(pl);
      break;
    case (container):
      write_text(pl, "Moving to Next Container Upwards");
      pl->itemmode--;
      if (pl->itemmode < 0)
      {
        pl->itemmode = 1;
        pl->top_held = pl->obj->holding;
      } else if (pl->itemmode == 0)
      {
        pl->top_held = pl->obj->below;
      } else
        pl->top_held = pl->top_held->owner;
      SET_PL_ITEM_CHANGED(pl);
      break;
    case (aggressive):
      if (PL_CHARGES(pl))
      {
        CLEAR_PL_CHARGES(pl);
        write_text(pl, "No longer in charging mode");
      } else
      {
        SET_PL_CHARGES(pl);
        write_text(pl, "You are now in Charging mode");
        write_text(pl, "You will attack anything you run into");
        write_text(pl, "when you are holding a melee type weapon");
      }
      break;
    case (parser):
      write_text(pl, "parser mode");
      pl->keystate = PARSE_STATE;
      pl->key_control = (&get_line);
      pl->px = 0;
      clear_parsing(pl);
      break;
    case (turn_left):
      pl->obj->facing = (pl->obj->facing + 6) % 8;
      break;
    case (turn_right):
      pl->obj->facing = (pl->obj->facing + 2) % 8;
      break;
    case (move_f):
      move_player(pl, pl->obj->facing);
      break;
    case (move_fr):
      move_player(pl, (pl->obj->facing + 1) % 8);
      break;
    case (move_r):
      move_player(pl, (pl->obj->facing + 2) % 8);
      break;
    case (move_br):
      move_player(pl, (pl->obj->facing + 3) % 8);
      break;
    case (move_b):
      move_player(pl, (pl->obj->facing + 4) % 8);
      break;
    case (move_bl):
      move_player(pl, (pl->obj->facing + 5) % 8);
      break;
    case (move_l):
      move_player(pl, (pl->obj->facing + 6) % 8);
      break;
    case (move_fl):
      move_player(pl, (pl->obj->facing + 7) % 8);
      break;
    case (apply_f):
      apply_bstate(pl, 0);
      break;
    case (apply_fr):
      apply_bstate(pl, 1);
      break;
    case (apply_r):
      apply_bstate(pl, 2);
      break;
    case (apply_br):
      apply_bstate(pl, 3);
      break;
    case (apply_b):
      apply_bstate(pl, 4);
      break;
    case (apply_bl):
      apply_bstate(pl, 5);
      break;
    case (apply_l):
      apply_bstate(pl, 6);
      break;
    case (apply_fl):
      apply_bstate(pl, 7);
      break;
    default:
      write_text(pl, "that key is not yet bound");
      break;
  }
}
void
get_simple_buttonpress(playerstruct * pl, XEvent event)
{
  SENDRQ(pl->button_object, recieve_button) (pl->button_object, pl->obj, &event.xbutton.window, &event.xbutton.button, &event.xbutton.x, &event.xbutton.y);
}

void
get_no_buttonpress(playerstruct * pl, XEvent event)
{
  return;
}

void
handle_button(playerstruct * pl, XEvent event)
{
}
