/*********************************************************/
/* file: obdata.c                                        */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 april 93                                 */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  if you add place your name here or below             */
/*                                                       */
/* description:                                          */
/*  obdata.c contains the lowest level subroutine calls  */
/* to manipulate objects. this includes creating and     */
/* destroying object, hashing them, linking them, etc... */
/* Many other encapsulation files rely on on this file   */
/* changes to this file must be made with care           */
/*                                                       */
/* LICENSING:                                            */
/*   a full license  is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, named LICENSE             */
/*********************************************************/
#include <obdata.h>

/*****************************************/
/* obj_hash                              */
/*  takes in a string and produces a hash*/
/* number in the range of 0 - 255;       */
/*****************************************/
int
name_hash(char *name,int size)
{
  int             hashnum = 0,
                  cur = 0;
  for (cur = 0; (name[cur] != '\0'); cur++)
    hashnum = hashnum * 2 + name[cur];
  hashnum = hashnum % size;
  return hashnum;
}

/*****************************************/
/*obj_rehash                             */
/*  takes in a string and produces a hash*/
/* number in the range of integers,  this*/
/* is done so that for hash collisions,  */
/* there is only an integer compare (~ 1 */
/* machine cycle) per collided object    */
/*****************************************/
int
name_rehash(char *name,int size)
{
  int             hashnum = 0,
                  cur = 0;

  for (cur = 0; (name[cur] != '\0'); cur++)
    hashnum = (hashnum * 3) ^ name[cur];
  return hashnum;
}
void
unlink_object(object * previous, object * current)
{
  current->references = 0;
  if (previous == NULL)
    switch (GET_TYPE(current))
    {
    case (player):
      if ((current->next != NULL) && IS_PLAYER(current->next))
      {
        players = current->next;
        if (current == playerend)
          playerend = current->next;
      } else
      {
        players = NULL;
        playerend = NULL;
      }
    default:
      list = current->next;
      if (list == NULL)
        list_end = NULL;
      break;
    }
  else
  {
    previous->next = current->next;
    if (current->next == NULL)
    {
      list_end = previous;
      if (IS_PLAYER(current))
        playerend = previous;
    } else if (IS_PLAYER(current) && !IS_PLAYER(current->next))
      playerend = previous;
  }
  current->next = pending_free;
  pending_free = current;
}

/**********************************************/
/* maintain_free                              */
/*    runs through list of pending free       */
/* objects and puts those objects with        */
/* references == 0 on to the free list, takes */
/* care of freeing the data segments of each  */
/* object                                     */
/**********************************************/
void
maintain_free()
{
  object         *previous,
                 *current,
                 *next,
                 *tmp,
                 *tmp_next;
  for (current = pending_free, previous = NULL; current != NULL;
       previous = ((current->references) == 0 ? previous : current),
       current = next)
  {
    next = current->next;
    if (!(current->references))
      {
	/* perhaps we should cue these also? */
	if (current->data.plcomp != NULL)
	  {
	    if (IS_PLAYER(current))
	      free (OB_PLDATA(current));
	    
	    free (current->data.emcomp);
	    
	    current->data.plcomp = NULL;
	  }
	
	/* free the linked objects that belong to this object */
	if (current->link != NULL)
	  {
	    for (tmp = current->link; tmp != NULL; tmp = tmp_next)
	      {
		SET_FREE(tmp);
		SET_REMOVED(tmp);
		tmp_next = tmp->link;
		tmp->next = free_obs;
		free_obs = tmp;
		tmp->link = NULL;
	      }
	    current->link = NULL;
	  }

	/* it's a kludge - one of the few in the program */
	/* I admit it.........  but it works!            */
	if (previous == NULL)
	  pending_free = next;
	else
	  previous->next = next;
	
	current->next = free_obs;
	free_obs = current;
      }
  }
}

/**********************************************/
/* add_object                                 */
/*  adds an object to the object proccessing  */
/* list, players go on at the front of list   */
/* and other objects go on just after the last*/
/* player on the list                         */
/**********************************************/
void
add_object(object * new)
{
  switch (new->type)
  {
    case (player):
    if (players == NULL)
    {
      players = new;
      playerend = new;
      new->next = NULL;
      if (list == NULL)
      {
        list = new;
        list_end = new;
      } else
      {
        new->next = list;
        list = new;
      }
    } else
    {
      if (playerend->next != NULL)
        new->next = playerend->next;
      playerend->next = new;

    fprintf(stderr, "\n \n %s \n \n", playerend->name);
    }
    break;
  default:
    if (list == NULL)
      list = new;
    else
      list_end->next = new;
    list_end = new;
    new->next = NULL;
    break;
  }
}

/*******************************************************/
/* free_object:                                        */
/* frees an object and sets it up to be cued on to the */
/* pending free list so that it may be reused later    */
/* immediatly cues general objects that turn up to be  */
/* freed to avoid memory leak                          */
/*******************************************************/

/**********************************************************/
/* perhaps in the future - this routine will call special */
/* free routines....                                      */
/**********************************************************/
void
free_object(object * obj)
{
  /* set up for cuing */
  SET_FREE(obj);

  /* make sure that it is not in the map */
  if (!IS_REMOVED(obj))
      {
	remove_ob(obj);
	SET_REMOVED(obj);
      }

  /* immediately cue general objects   */
  if ((obj->gen_ob == NULL) || (obj->type == BAD))
  {
    obj->next = free_obs;
    free_obs = obj;
  }
}

/**********************************************/
/* player_alloc                               */
/* player allocation subroutine to initialize */
/* a new objects data component               */
/**********************************************/
void
player_alloc(object * ob, playerd * copy)
{
  playerd        *tempplayer;

  tempplayer = (playerd *) calloc(1, sizeof(playerd));
  if (copy != NULL)
    memcpy(tempplayer, copy, sizeof(playerd));
  tempplayer->pldata = (playerstruct *) calloc(1, sizeof(playerstruct));
  ob->data.plcomp = tempplayer;
}

/*********************************************/
/* free_gcs                                  */
/* free all the graphic contexts used by in  */
/* a particular window -used by player_free  */
/*********************************************/
void
free_gcs (playerstruct *pl,drawdata *win)
{

  XFreeGC (pl->display,win->itembg_gc);
  XFreeGC (pl->display,win->itemfg_gc);
  XFreeGC (pl->display,win->itemxor_gc);
  XFreeGC (pl->display,win->text_gc);
}

/********************************************/
/* player_free                              */
/* routine that must be called to cleanup   */
/* the data component of a player that no   */
/* no longer exists                         */
/********************************************/
void
player_free(object * ob)
{
  int             count;
  keystruct      *t_key,* key_next;
  playerstruct   *pl = OB_PLDATA(ob);

  for (count = 0; count < CURSORMAX; count++)
    XFreeCursor(pl->display, pl->pointers[count]);

  for (t_key = pl->keys; t_key != NULL; t_key = key_next)
    {
      key_next = t_key->next;
      free (t_key);
    }

  XFreeFont(pl->display, pl->itemfont);
  XFreeFont(pl->display, pl->textfont);

  free_gcs (pl,&pl->main);
  free_gcs (pl,&pl->viewpix);
  free_gcs (pl,&pl->itempix);
  free_gcs (pl,&pl->item);
  free_gcs (pl,&pl->stats);
  free_gcs (pl,&pl->controls);
  free_gcs (pl,&pl->text);
  free_gcs (pl,&pl->view);
  
  for (count =1; count < NUM_PICS; count ++)
    {
      XFreeGC (pl->display,pl->maskgcs[count]);
      XFreePixmap (pl->display,pl->masks[count]);
    }

  XFreePixmap(pl->display, pl->viewpix.window);
  XFreePixmap(pl->display, pl->masks[0]);
}
/*********************************************/
/* remove_ob                                 */
/*    unlinks an object from it current map  */
/*  binding, changes objects data structure  */
/*  and maps data structure to reflect the   */
/*  changes                                  */
/*********************************************/
void
remove_ob(object * obj)
{
  object         *tmp,
                 *curr;
  
  /* operate starting on the head object */
  obj = HEAD(obj);
  
  /* do not attempt to remove if already removed or not in a map */
  if (obj->mapin == NULL || OB_IS_REMOVED(obj))
    return;
 
  /* run through all links of this object */
  for (curr = obj; curr != NULL; curr = curr->link)
    {
      /* if it is a floor, find next topmost floor object */
      /* and set the tile pointer to the found object     */
      if (curr->type == floor && (obj->z == 0))
	{
	  for (tmp = curr->below; tmp != NULL && GET_TYPE(tmp) != floor;
	       tmp = tmp->below);
	  MAP_FLOOR_OBJ(curr->mapin, obj->x, obj->y) = tmp;
	}
      
      /* fixe the below pointer */
      if (curr->below != NULL)
	curr->below->above = obj->above;
      else
	BOT_MAP_OBJ(curr->mapin, curr->x, curr->y, curr->z) = curr->above;
      
      /* fix the above pointer */
      if (curr->above != NULL)
	curr->above->below = curr->below;
      else
	TOP_MAP_OBJ(curr->mapin, curr->x, curr->y, curr->z) = curr->below;
      
      /* make sure the object is not linked to anything in a stack */
      curr->above = curr->below = NULL;
    }
  /* for other objects reference */
  SET_REMOVED(obj);
}

/*******************************************************/
/* free_player:                                        */
/* frees the object pointed to by the passed           */
/* playerstruct structure, performs cleanup actions    */
/* neccessary for a playerstruct                       */
/*******************************************************/
void
free_player(playerstruct * pl)
{
  (void) player_free(pl->obj);
  XCloseDisplay(pl->display);
  free_object(pl->obj);
}


/*******************************************************/
/* anim_alloc                                          */
/*   allocate the data portion of an animate object    */
/*******************************************************/
void
anim_alloc(object * ob, animd * copy)
{
  ob->data.ancomp = (animd *) (calloc(1, sizeof(animd)));
  if (copy != NULL)
    memcpy(ob->data.ancomp, copy, sizeof(animd));

}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
anim_free(object * ob)
{
}

/*******************************************************/
/* inanim_alloc                                        */
/*   allocate the data portion of an inanimate object  */
/*******************************************************/
void
inanim_alloc(object * ob, inanimd * copy)
{
  ob->data.incomp = (inanimd *) (calloc(1, sizeof(inanimd)));
  if (copy != NULL)
    memcpy(ob->data.incomp, copy, sizeof(inanimd));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
inanim_free(object * ob)
{
}

/*******************************************************/
/* tele_alloc                                          */
/*   allocate the data portion of a telporter object   */
/*******************************************************/
void
tele_alloc(object * ob, teled * copy)
{
  ob->data.tecomp = (teled *) (calloc(1, sizeof(teled)));
  if (copy != NULL)
    memcpy(ob->data.tecomp, copy, sizeof(teled));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
tele_free(object * ob)
{
}

/*******************************************************/
/* armour_alloc                                        */
/*   allocate the data portion of an amour object      */
/*******************************************************/
void
armour_alloc(object * ob, armourd * copy)
{
  ob->data.arcomp = (armourd *) (calloc(1, sizeof(armourd)));
  if (copy != NULL)
    memcpy(ob->data.arcomp, copy, sizeof(armourd));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
armour_free(object * ob)
{
}

/*******************************************************/
/* spell_alloc                                         */
/*   allocate the data portion of a spell object       */
/*******************************************************/
void
spell_alloc(object * ob, spelld * copy)
{
  ob->data.spcomp = (spelld *) (calloc(1, sizeof(spelld)));
  if (copy != NULL)
    memcpy(ob->data.spcomp, copy, sizeof(spelld));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
spell_free(object * ob)
{
}

/*******************************************************/
/* weapon_alloc                                        */
/*   allocate the data portion of a weapon object      */
/*******************************************************/
void
weapon_alloc(object * ob, weapond * copy)
{
  ob->data.wecomp = (weapond *) (calloc(1, sizeof(weapond)));
  if (copy != NULL)
    memcpy(ob->data.wecomp, copy, sizeof(weapond));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
weapon_free(object * ob)
{
}

/*******************************************************/
/* ability_alloc                                       */
/*   allocate the data portion of an ability object    */
/*******************************************************/
void
ability_alloc(object * ob, abilityd * copy)
{
  ob->data.abcomp = (abilityd *) (calloc(1, sizeof(abilityd)));
  if (copy != NULL)
    memcpy(ob->data.abcomp, copy, sizeof(abilityd));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
ability_free(object * ob)
{
}

/*******************************************************/
/* empyy_alloc                                         */
/*   allocate the data portion of an empty object      */
/*******************************************************/
void
empty_alloc(object * ob, emptyd * copy)
{
  ob->data.emcomp = (emptyd *) (calloc(1, sizeof(emptyd)));
  if (copy != NULL)
    memcpy(ob->data.emcomp, copy, sizeof(emptyd));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
empty_free(object * ob)
{
}

/*******************************************************/
/* wall_alloc                                          */
/*   allocate the data portion of a wall object        */
/*******************************************************/
void
wall_alloc(object * ob, walld * copy)
{
  ob->data.wacomp = (walld *) (calloc(1, sizeof(walld)));
  if (copy != NULL)
    memcpy(ob->data.wacomp, copy, sizeof(walld));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
wall_free(object * ob)
{
}

/*******************************************************/
/* floor_alloc                                         */
/*   allocate the data portion of a floor object       */
/*******************************************************/
void
floor_alloc(object * ob, floord * copy)
{
  ob->data.flcomp = (floord *) (calloc(1, sizeof(floord)));
  if (copy != NULL)
    memcpy(ob->data.flcomp, copy, sizeof(floord));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
floor_free(object * ob)
{
}

/*******************************************************/
/* inanim_alloc                                        */
/*   allocate the data portion of a telporter object   */
/*******************************************************/
void
switch_alloc(object * ob, linkerd * copy)
{
  ob->data.licomp = (linkerd *) (calloc(1, sizeof(linkerd)));
  if (copy != NULL)
    memcpy(ob->data.licomp, copy, sizeof(linkerd));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
switch_free(object * ob)
{
}

/*******************************************************/
/* inanim_alloc                                        */
/*   allocate the data portion of a telporter object   */
/*******************************************************/
void
door_alloc(object * ob, doord * copy)
{
  ob->data.docomp = (doord *) (calloc(1, sizeof(doord)));
  if (copy != NULL)
    memcpy(ob->data.docomp, copy, sizeof(doord));
}

/*******************************************************/
/* reserved for future use                             */
/*******************************************************/
void
door_free(object * ob)
{
}


/***********************************************/
/* add_link                                    */
/*   will add an object to another objects     */
/* list of objects to be linked as part of the */
/* same object                                 */
/***********************************************/
void add_link (object *head, object *linked)
{
  object *tmp;
  
  for (tmp = head ; tmp->link != NULL; tmp= tmp->link);
  
  tmp->link = linked;
  linked->next = NULL;
  linked->holding = head;
  linked->facing = head->facing;    
}

/***************************************/
/* find_in_hash                        */
/*  will find and return the pointer to*/
/* the object with the given name in   */
/* the object hash table               */
/***************************************/
hashed_object  *
find_in_hash(char *name)
{
  hashed_object  *tmp;
  int             i,
                  j;

  i = name_hash(name,OBJECT_HASH_SIZE);
  j = name_rehash(name,OBJECT_HASH_SIZE);
  for (tmp = obdata[i]; (tmp != NULL) && (tmp->rehash != j); tmp = tmp->next);

  return tmp;
}

/**************************************/
/* get_free_ob                        */
/*   will deque an object from the    */
/* free_objects list and return it    */
/**************************************/
object         *
get_free_ob()
{
  object         *temp;
  temp = free_obs;
  free_obs = temp->next;
  return (temp);
}


/**************************************/
/* get_blank_object                   */
/*   will use malloc to create a new  */
/* object and memset to initialize    */
/* all entries to 0/NULL              */
/**************************************/
object         *
get_blank_object(object_type type)
{
  object         *new_ob;
  if (free_obs == NULL)
    new_ob = (object *) malloc(sizeof(object));
  else
    new_ob = get_free_ob();
  memset(new_ob, 0, sizeof(object));
  new_ob->state.normal = new_ob->state.input = 0;
  ob_alloc[type] (new_ob, NULL);
  CLEAR_FREE(new_ob);
  return new_ob;
}


/****************************************/
/* get_general_object                   */
/*   given a name, this procedure will  */
/* look it up in the object hash table  */
/* and return the general instance of   */
/* it if it is there                    */
/****************************************/
object         *
get_general_object(char *name)
{

  object         *new_ob,*head,*copy;
  hashed_object  *tmp;

  tmp = find_in_hash(name);
  
  if (tmp == NULL)
    return NULL;
    
  if (free_obs == NULL)
      new_ob = head = (object *) malloc(sizeof(object));
  else
      new_ob = head = get_free_ob();
  
  copy =  tmp->obj;
 
  while (copy!=NULL)
    {         
      ob_alloc[GET_TYPE(tmp->obj)] (new_ob, tmp->obj->data.emcomp);
      CLEAR_FREE(new_ob);
      (void) memcpy(new_ob, copy, sizeof(object) - 4);
      if (new_ob != head)
	new_ob->holding = head;
      SET_REMOVED(new_ob);
      
      new_ob->speed_left = 0;
      new_ob->gen_ob = tmp->obj;
      new_ob->references = 0;
      
      copy = copy->link;
      
      if (copy != NULL)
	{
	  if (free_obs == NULL)
	    new_ob->link = (object *) malloc(sizeof(object));
	  else
	    new_ob->link = get_free_ob();
	}
      new_ob = new_ob->link;
    }
  
/* a quick hack - soon to be fixed in add_object */
  head->speed_left = -GET_SPEED(head);
  add_object(head);   
  return head;
}


/*********************************************/
/* duplicate_object                          */
/*   given an object, this procedure will    */
/* produce a perfect copy of that object and */
/* all of its belongings                     */
/*********************************************/
object          *
duplicate_object (object *ob)
{
  object *new_ob,
         *curr,
          *tmp;
  
  if (ob == NULL)
    return NULL;
  
  if (free_obs == NULL)
      new_ob = (object *)malloc(sizeof(object));
  else
      new_ob = get_free_ob();
  memcpy (new_ob,ob,sizeof(object)-4);
  SET_REMOVED(new_ob);
  new_ob->above = new_ob->below = new_ob->owner 
    = new_ob->holding = new_ob->link = NULL;
  new_ob->references = 0;
  /* remember that names are in a shared/hashed structure!*/
  SET_NAME(new_ob,GET_NAME(ob));

  add_object (new_ob);
  ob_alloc[GET_TYPE(ob)](new_ob,ob->data.emcomp);

  for (curr = ob->link; curr != NULL; curr = curr->link)
  {
    if (free_obs == NULL)
      tmp = (object *)malloc(sizeof(object));
    else
      tmp = get_free_ob();
    memcpy(tmp,curr,sizeof(object)-4);
    add_link (new_ob,tmp);
  }

  if (HAS_ITEMS(ob))
    for (tmp = ob->holding; tmp != NULL ; tmp = tmp->below)
      add_holding (ob,duplicate_object (tmp));
  return (new_ob);
}


/* temporary until player read/write functions are complete */
object         *
readplayer(char *name)
{
  object         *tmp;
  FILE           *playerfile;
  char            fname[100];

  strcpy(fname, LIBDIR);
  strcat(fname, "/playerfiles/");
  strcat(fname, name);
  if ((playerfile = fopen(fname, "r")))
  {
    tmp = readobject(playerfile, MODE_NEW);
    fclose(playerfile);
  } else
  {
    fprintf(stderr, "players read error %s\n", fname);
    return (NULL);
  }
  return tmp;
}


/***************************************/
/* add_to_hashlist                     */
/*   given an object, this procedure   */
/* will add it to the hashlist as  a   */
/* general object to be retrieved later*/
/* by calls to get_general_object      */
/***************************************/
void
add_to_hashlist(object * tmpob)
{
  hashed_object  *tloc,
                 *curr;
  int             hashlock;
  char            tmpstr[128];

  if ((tloc = find_in_hash(tmpob->name)) != NULL)
  {
    fprintf(stderr, "\tattempted to re-add %s to hashtable\n", tmpob->name);
    get_char_statbits(tmpstr, tloc->obj);
    fprintf(stderr, "\tobject data includes name :%s type:%s, statbits:%s",
          tloc->obj->name, type_names[tloc->obj->type], tmpstr);
    return;
  }

  hashlock = name_hash(tmpob->name,OBJECT_HASH_SIZE);

  tloc = (hashed_object *) malloc(sizeof(hashed_object));
  tloc->obj = tmpob;
  tloc->rehash = name_rehash(tmpob->name,OBJECT_HASH_SIZE);

  if (obdata[hashlock] == NULL)
  {
    obdata[hashlock] = tloc;
    obdata[hashlock]->next = NULL;
  } else
  {
    for (curr = obdata[hashlock]; curr->next != NULL;
         curr = curr->next);
    curr->next = tloc;
    curr->next->next = NULL;
  }
}

/******************************************/
/* remove_from_hashlist                   */
/*   given a name, this procedure will    */
/* search for the object with that name   */
/* in the hash_list and remove it         */
/******************************************/
void
remove_from_hashlist(char *name)
{
  hashed_object **tloc,
                **curr;
  int             hashnum,
                  rehash;

  hashnum = name_hash(name,OBJECT_HASH_SIZE);
  rehash = name_rehash(name,OBJECT_HASH_SIZE);

  for (curr = &obdata[hashnum]; ((*curr)->rehash != rehash) && (*curr != NULL)
       ; curr = &((*curr)->next));
  /**************************************/
  /* the object itself may be           */
  /* neccesary for future calculations! */
  /* do NOT free the object             */
  /**************************************/
  /*free((*curr)->obj);*/
  tloc = &((*curr)->next);
  free(*curr);
  *curr = *tloc;
}

/*******************************************/
/* find_name                               */
/*   given a name, this procedure will     */
/* attempt to find the name in the name    */
/* hashlist, returning it if it is found   */
/* and null if not                         */
/*******************************************/
hashed_name    *
find_name(char *name)
{
  hashed_name    *tmp;
  int             i,
                  j;

  if (name == NULL)
    return NULL;
  i = name_hash(name,NAME_HASH_SIZE);
  j = name_rehash(name,NAME_HASH_SIZE);
  for (tmp = names[i];
       tmp != NULL && tmp->rehash != j;
       tmp = tmp->next);
  return tmp;
}

/*************************************/
/* add_holding                       */
/*    add_holding adds an item to the*/
/* inventory of the object specified */
/* by holder argument. it sets all   */
/* the pointers neccessary to reflect*/
/* the new situation                 */
/*************************************/
void
add_holding(object * holder, object * item)
{
  if (!OB_IS_REMOVED(item))
  {
    fprintf(stderr, "tried to get non-removed object");
    return;
  }
  item->below = holder->holding;
  if (holder->holding != NULL)
    holder->holding->above = item;
  holder->holding = item;
  item->owner = holder;
  holder->references++;
}

/*******************************************/
/* remove_held                             */
/*    remove_held removes the specified    */
/* object from the owners inventory, and   */
/* sets the pointers correctly for both    */
/* for the new situation where the item is */
/* separated from the owner                */
/*******************************************/
void
remove_held(object * item)
{
  if (item->owner->holding == item)
    item->owner->holding = item->below;
  else
    item->above->below = item->below;

  /* players need to have their item window changed! */
  if (IS_PLAYER(item->owner))
  {
    if (item == OB_PLDATA(item->owner)->top_held)
      OB_PLDATA(item->owner)->top_held = item->below;
    SET_PL_ITEM_CHANGED(OB_PLDATA(item->owner));
  }
  /* link the item that is below this one in the inventory to */
  /* the item that was above this one                         */
  if (item->below != NULL)
    item->below->above = item->above;

  /* decreas the holder reference count */
  item->owner->references--;

  /* These are no longer valid , so set them all to NULL */
  item->below = item->above = item->owner = NULL;
}


/*******************************************/
/* set_obj_name                            */
/*   this procedure sets a given objects   */
/* name to the supplied string, adding the */
/* supplied string to the name hash list   */
/* if it is not alreadyu there             */
/*******************************************/
void
set_obj_name(object * obj, char *name)
{
  hashed_name    *tmp,
                 *curr;
  int             hashlock;

  if ((tmp = find_name(GET_NAME(obj))) != NULL)
  {
    tmp->references--;
  }
  if ((tmp = find_name(name)) != NULL)
  {
    tmp->references++;
    obj->name = tmp->name;
  }
  hashlock = name_hash(name,NAME_HASH_SIZE);
  tmp = (hashed_name *) malloc(sizeof(hashed_name));
  strncpy(tmp->name, name, 32);
  obj->name = tmp->name;

  tmp->rehash = name_rehash(name,NAME_HASH_SIZE);

  if (names[hashlock] == NULL)
  {
    names[hashlock] = tmp;
    tmp->next = NULL;
  } else
  {
    for (curr = names[hashlock]; curr->next != NULL; curr = curr->next);
    curr->next = tmp;
    curr->next->next = NULL;
  }
}

/*****************************************/
/* set_message                           */
/*   this subroutine will set the given  */
/* objects message to the supplied string*/
/* by appending a message structure to   */
/* the object                            */
/*****************************************/
void
set_message (object *ob,char *message)
{
  msg *tmp_msg,*current;
  
  tmp_msg = (msg *) malloc(sizeof (msg));

  tmp_msg->text = (char *) malloc (strlen(message));
  
  strcpy(tmp_msg->text,message);

  tmp_msg->next = NULL;

  if (ob->message == NULL)
    ob->message = tmp_msg;
  else
    {
      for(current = ob->message;
	  current-> next != NULL;
	  current = current->next);

      current->next = tmp_msg;
      
    }
}

/******************************************/
/* free_all_obj_messages                  */
/*   this procedure will remove all       */
/* messages associated with the given     */
/* object                                 */
/******************************************/
void
free_all_obj_messages (object *ob)
{
  msg *curr,*next;
  
  if (ob->message == NULL)
    return;
  
  for (curr = ob->message; curr != NULL; curr = next)
    {
      next = curr;
      free (curr->text);
      free (curr);
    }
}

/*********************************************/
/* finde_channel                             */
/*   this procedure will find the channel -  */
/* head structure in the channel head hash   */
/* table if it exists and returns it, if it  */
/* does not exist - it returns NULL          */
/*********************************************/
channel_head *
find_channel (char *channel_name)
{
  int hash_loc, rehash;
  
  channel_head *t_head;
  hash_loc = name_hash (channel_name,CHANNEL_HASH_SIZE);
  rehash = name_rehash (channel_name,CHANNEL_HASH_SIZE);
  
  for (t_head = channel_table[hash_loc];
       t_head != NULL && t_head->rehash != rehash;
       t_head = t_head->next);
  
  return t_head;
}

/********************************************/
/* find_add_channel                         */
/*   this procedure will find the channel - */
/* head structure in the channel head hash  */
/* table if it exists and returns it, if it */
/* does not exist, it will create a new     */
/* channel and return the new channel       */
/********************************************/
channel_head *
find_add_channel (char *channel_name)
{
  channel_head *t_head,*curr;
  int hash_loc, rehash;
  
  if ((t_head = find_channel (channel_name))==NULL)
    {
      hash_loc = name_hash (channel_name,CHANNEL_HASH_SIZE);
      rehash = name_rehash (channel_name,CHANNEL_HASH_SIZE);
      t_head = (channel_head *) malloc (sizeof(channel_head));
      t_head->refs = 0;
      t_head->rehash = rehash;
      t_head->entries = NULL;
      t_head->next = NULL;
      
      strcpy (t_head->name,channel_name);
      
      if (channel_table [hash_loc] == NULL)
	channel_table[hash_loc] = t_head;
      else
	{
	  for (curr = channel_table [hash_loc];
	       curr->next != NULL;
	       curr = curr->next);
	  curr->next = t_head;
	}
    }
  return t_head;
}

/**********************************************/
/* add_channel_entry                          */
/*   this procedure will add a new channel    */
/* entry to the supplied object               */
/**********************************************/
channel_entry *
add_channel_entry (channel_head *head,object *obj)
{
  channel_entry *new_entry;

  new_entry = (channel_entry *) malloc(sizeof(channel_entry));
  new_entry->obj = obj;
  
  
  new_entry->prev = NULL;
  
  new_entry->next = head->entries;

  if (head->entries != NULL)
    head->entries->prev = new_entry;
    
  head->entries = new_entry;

  return new_entry;
}

void
add_channel_to_obj (object *ob,char *channel_name)
{
  channel_ref   *tmp_ref;
  channel_head  *tmp_head;

  tmp_ref = (channel_ref *) malloc (sizeof (channel_ref));
  
  tmp_head = find_add_channel (channel_name);
  
  tmp_ref->head = tmp_head;
  
  tmp_ref->entry = add_channel_entry(tmp_head,ob);
  
  tmp_ref->next = ob->channels;
  
  ob->channels = tmp_ref;
}

void 
free_all_obj_channels(object *ob)
{
  channel_ref *curr,*next;
  
  for (curr = ob->channels; curr != NULL; curr = next)
    {
      next = curr;
      
      if (curr->entry->prev != NULL)
	curr->entry->prev->next = curr->entry->next;
      
      if (curr->entry->next != NULL)
	curr->entry->next->prev = curr->entry->prev;

      if (curr->head->entries == curr->entry)
	curr->head->entries = curr->entry->next;

      free (curr->entry);
	
      curr->head->refs --;
      
      free (curr);
    }
}
