/*********************************************************/
/* file: obset.c                                         */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 april 93                                 */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  if you add place your name here or below             */
/*                                                       */
/* description:                                          */
/*  obset.c contains subroutines to set and extract data */
/* from objects in the form of strings. These routines   */
/* are particularily useful for wset, iset and file I/O  */
/* and are hopefully in a form that is not difficult for */
/* humans to decifer                                     */
/*                                                       */
/* LICENSING:                                            */
/*   a full license  is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, named LICENSE             */
/*********************************************************/
#include <standard.h>
 
/******************************************/
/* set_char_statbits                      */
/*  takes in a string representing which  */
/* statbits to assert and deny and sets   */
/* the statbits of the passed object      */
/* according to that string               */
/******************************************/
void
set_char_statbits(char *statstr, object * ob)
{
	int             count,
	max;
	max = strlen(statstr);
	for (count = 0; count < max; count++)
	  switch (statstr[count])
	    {
	    case ('P'):
	      SET_PHASED(ob);
	      break;
	    case ('p'):
	      CLEAR_PHASED(ob);
	      break;
	    case ('V'):
	      SET_BLOCKS_VIEW(ob);
	      break;
	    case ('v'):
	      CLEAR_BLOCKS_VIEW(ob);
	      break;
	    case ('M'):
	      SET_BLOCKS_MOVE(ob);
	      break;
	    case ('m'):
	      CLEAR_BLOCKS_MOVE(ob);
	      break;
	    case ('T'):
	      SET_CAN_TAKE(ob);
	      break;
	    case ('t'):
	      CLEAR_CAN_TAKE(ob);
	      break;
	    case ('D'):
	      SET_CAN_DROP(ob);
	      break;
	    case ('d'):
	      CLEAR_CAN_DROP(ob);
	      break;
	    case ('E'):
	      SET_IS_EQUIPPED(ob);
	      break;
	    case ('e'):
	      CLEAR_IS_EQUIPPED(ob);
	      break;
	    case ('S'):
	      SET_NEEDS_STATE(ob);
	      break;
	    case ('s'):
	      CLEAR_NEEDS_STATE(ob);
	      break;
	    default:
	      fprintf(stderr, "attempt to set unknown statbit in %s\n", 
		      GET_NAME(ob));
	      break;
	    }
      }

/******************************************/
/* set_obj_material                       */
/*  takes in a string representing which  */
/* materials to assert and deny and sets  */
/* the material of the passed object      */
/* according to that string               */
/******************************************/
void
set_obj_material(char *matstr, object * obj)
{
  int             count,
                  max;
  
  max = strlen(matstr);
  for (count = 0; count < max; count++)
    {
      switch (matstr[count])
	{
	case ('W'):
	  obj->material.wood = 1;
	  break;
	case ('w'):
	  obj->material.wood = 0;
	  break;
	case ('H'):
	  obj->material.hardwood = 1;
	  break;
	case ('h'):
	  obj->material.hardwood = 0;
	  break;
	case ('P'):
	  obj->material.plastic = 1;
	  break;
	case ('p'):
	  obj->material.plastic = 0;
	  break;
	case ('U'):
	  obj->material.rubber = 1;
	  break;
	case ('u'):
	  obj->material.rubber = 0;
	  break;
	case ('G'):
	  obj->material.glass = 1;
	  break;
	case ('g'):
	  obj->material.glass = 0;
	  break;
	case ('A'):
	  obj->material.paper = 1;
	  break;
	case ('a'):
	  obj->material.paper = 0;
	  break;
	case ('I'):
	  obj->material.mineral = 1;
	  break;
	case ('i'):
	  obj->material.mineral = 0;
	  break;
	case ('M'):
	  obj->material.metal = 1;
	  break;
	case ('m'):
	  obj->material.metal = 0;
	  break;
	case ('O'):
	  obj->material.organix = 1;
	  break;
	case ('o'):
	  obj->material.organix = 0;
	  break;
	case ('N'):
	  obj->material.energy = 1;
	  break;
	case ('n'):
	  obj->material.energy = 0;
	  break;
	case ('E'):
	  obj->material.ether = 1;
	  break;
	case ('e'):
	  obj->material.ether = 0;
	  break;
	case ('S'):
	  obj->material.synthetics = 1;
	  break;
	case ('s'):
	  obj->material.synthetics = 0;
	  break;
	default:
	  fprintf(stderr, "unknown composition var. in object");
	  break;
	}
    }
}

/******************************************/
/* set_obj_resist                         */
/*  takes in a string representing which  */
/* resistances to assert and deny and sets*/
/* the resistances of the passed object   */
/* according to that string               */
/******************************************/
void
set_obj_resist(char *restring, object * obj)
{
  int             count,
	          max;

  max = strlen(restring);
  for (count = 0; count < max; count += 2)
    {
      switch (restring[count])
	{
	case ('h'):
	  obj->resist.heat = -(restring[count + 1] - '0');
	  break;
	case ('H'):
	  obj->resist.heat = (restring[count + 1] - '0');
	  break;
	case ('c'):
	  obj->resist.cold = -(restring[count + 1] - '0');
	  break;
	case ('C'):
	  obj->resist.cold = (restring[count + 1] - '0');
	  break;
	case ('e'):
	  obj->resist.electric = -(restring[count + 1] - '0');
	  break;
	case ('E'):
	  obj->resist.electric = (restring[count + 1] - '0');
	  break;
	case ('p'):
	  obj->resist.physical = -(restring[count + 1] - '0');
	  break;
	case ('P'):
	  obj->resist.physical = (restring[count + 1] - '0');
	  break;
	case ('m'):
	  obj->resist.mental = -(restring[count + 1] - '0');
	  break;
	case ('M'):
	  obj->resist.mental = (restring[count + 1] - '0');
	  break;
	case ('o'):
	  obj->resist.poison = -(restring[count + 1] - '0');
	  break;
	case ('O'):
	  obj->resist.poison = (restring[count + 1] - '0');
	  break;
	case ('r'):
	  obj->resist.radiation = -(restring[count + 1] - '0');
	  break;
	case ('R'):
	  obj->resist.radiation = (restring[count + 1] - '0');
	  break;
	case ('a'):
	  obj->resist.acid = -(restring[count + 1] - '0');
	  break;
	case ('A'):
	  obj->resist.acid = (restring[count + 1] - '0');
	  break;
	default:
	  fprintf(stderr,"unknown resistanc var. in object %s\n",
		  GET_NAME(obj));
	  break;
	}
    }
}

/******************************************/
/* set_obj_usage                          */
/*  takes in a string representing which  */
/* usages to assert and deny and sets     */
/* the usage of the passed object         */
/* according to that string               */
/******************************************/
void
set_obj_usage (char *usestring, object *obj)
{
  int             count,
	          max;
  
  max = strlen(usestring);
  for (count = 0; count < max; count++)
    {
      switch (usestring[count])
	{
	case ('M'):
	  obj->usage.mystic = 1;
	  break;
	case ('m'):
	  obj->usage.mystic = 0;
	  break;
	case ('D'):
	  obj->usage.disciple = 1;
	  break;
	case ('d'):
	  obj->usage.disciple = 0;
	  break;
	case ('N'):
	  obj->usage.ninja = 1;
	  break;
	case ('n'):
	  obj->usage.ninja = 0;
	  break;
	case ('B'):
	  obj->usage.brute = 1;
	  break;
	case ('b'):
	  obj->usage.brute = 0;
	  break;
	case ('W'):
	  obj->usage.warrior = 1;
	  break;
	case ('w'):
	  obj->usage.warrior = 0;
	  break;
	case ('T'):
	  obj->usage.treach = 1;
	  break;
	case ('t'):
	  obj->usage.treach = 0;
	  break;
	case ('S'):
	  obj->usage.sniper = 1;
	  break;
	case ('s'):
	  obj->usage.sniper = 0;
	  break;
	case ('O'):
	  obj->usage.none = 1;
	  break;
	case ('o'):
	  obj->usage.none = 0;
	  break;
	}
    }
}

void
set_obj_bonus(char *bonusstring, object * obj)
{
  int             count,
                  value,
                  max;
  
  char            bonustype,
                  next[128],
  
  *curr = bonusstring;
  
  max = strlen(bonusstring);
  
  for (count = 0; count < 4 ; count ++, curr = next)
    {
      if(sscanf(curr,"%c%d%s",&bonustype,&value,next) < 2)
	{
	  fprintf (stderr,"error in bonus string for %s\n",GET_NAME(obj));
	  fprintf (stderr," count %d c %c d %d s %s\n",count,bonustype,
		   value,next);
	  return;
	}
      obj->data.arcomp->bonuses[count].amount = value;
      switch (bonustype)
	{
	case ('n'):
	case ('N'):
	  obj->data.arcomp->bonuses[count].type = nobonus;
	  break;
	case ('s'):
	case ('S'):
	  obj->data.arcomp->bonuses[count].type = str;
	  break;
	case ('i'):
	case ('I'):
	  obj->data.arcomp->bonuses[count].type = sma;
	  break;
	case ('w'):
	case ('W'):
	  obj->data.arcomp->bonuses[count].type = wis;
	  break;
	case ('d'):
	case ('D'):
	  obj->data.arcomp->bonuses[count].type = dex;
	  break;
	case ('c'):
	case ('C'):
	  obj->data.arcomp->bonuses[count].type = con;
	  break;
	case ('u'):
	case ('U'):
	  obj->data.arcomp->bonuses[count].type = luck;
	  break;
	case ('l'):
	case ('L'):
	  obj->data.arcomp->bonuses[count].type = looks;
	  break;
	case ('h'):
	case ('H'):
	  obj->data.arcomp->bonuses[count].type = cha;
	  break;
	case ('p'):
	case ('P'):
	  obj->data.arcomp->bonuses[count].type = speed;
	  break;
	default:
	  fprintf (stderr,"unknown bonus in %s",GET_NAME(obj));
	  return;
	}
    }
}

void
get_char_statbits(char *ret_string, object * ob)
{
  int             strloc = 0;
  
  if (PHASED(ob))
    ret_string[strloc++] = 'P';
  else
    ret_string[strloc++] = 'p';
  
  if (BLOCKS_VIEW(ob))
    ret_string[strloc++] = 'V';
  else
    ret_string[strloc++] = 'v';

  if (BLOCKS_MOVE(ob))
    ret_string[strloc++] = 'M';
  else
    ret_string[strloc++] = 'm';
  
  if (CAN_DROP(ob))
    ret_string[strloc++] = 'D';
  else
    ret_string[strloc++] = 'd';
  
  if (CAN_TAKE(ob))
    ret_string[strloc++] = 'T';
  else
    ret_string[strloc++] = 't';
  
  ret_string[strloc] = '\0';
}

void
get_obj_material(char *retstring, object * obj)
{
  int             strloc = 0;
  
  if (obj->material.wood)
    retstring[strloc++] = 'W';
  if (obj->material.hardwood)
    retstring[strloc++] = 'H';
  if (obj->material.plastic)
    retstring[strloc++] = 'P';
  if (obj->material.rubber)
    retstring[strloc++] = 'U';
  if (obj->material.glass)
    retstring[strloc++] = 'G';
  if (obj->material.paper)
    retstring[strloc++] = 'A';
  if (obj->material.mineral)
    retstring[strloc++] = 'I';
  if (obj->material.metal)
    retstring[strloc++] = 'M';
  if (obj->material.organix)
    retstring[strloc++] = 'O';
  if (obj->material.energy)
    retstring[strloc++] = 'N';
  if (obj->material.ether)
    retstring[strloc++] = 'E';
  retstring[strloc++] = '\0';
}

void get_obj_resist(char* retstring,object *obj)
{
  int					strloc=0;
  
  retstring[strloc ++]= (obj->resist.heat <0 ? 'h': 'H');
  retstring[strloc ++]= abs (obj->resist.heat) + '0'; 
  retstring[strloc ++]= (obj->resist.cold <0 ? 'c': 'C');
  retstring[strloc ++]= abs (obj->resist.cold)+ '0'; 
  retstring[strloc ++]= (obj->resist.electric <0 ? 'e': 'E');
  retstring[strloc ++]= abs (obj->resist.electric)+ '0'; 
  retstring[strloc ++]= (obj->resist.physical <0 ? 'p': 'P');
  retstring[strloc ++]= abs (obj->resist.physical)+ '0'; 
  retstring[strloc ++]= (obj->resist.mental <0 ? 'm': 'M');
  retstring[strloc ++]= abs (obj->resist.mental)+ '0'; 
  retstring[strloc ++]= (obj->resist.poison <0 ? 'o': 'O');
  retstring[strloc ++]= abs (obj->resist.poison)+ '0'; 
  retstring[strloc ++]= (obj->resist.radiation <0 ? 'r': 'R');
  retstring[strloc ++]= abs (obj->resist.radiation)+ '0';
  retstring[strloc ++]= (obj->resist.acid <0 ? 'a': 'A');
  retstring[strloc ++]= abs (obj->resist.acid)+ '0';
  retstring[strloc++] = '\0'; 
}

void get_obj_usage(char *retstring,object *obj)
{
  int		strloc = 0;
  
  retstring[strloc ++] = (obj->usage.mystic ? 'M':'m');
  retstring[strloc ++] = (obj->usage.disciple ? 'D':'d');
  retstring[strloc ++] = (obj->usage.ninja ? 'N':'n');
  retstring[strloc ++] = (obj->usage.brute ? 'B':'b');
  retstring[strloc ++] = (obj->usage.warrior ? 'W':'w');
  retstring[strloc ++] = (obj->usage.treach ? 'T':'t');
  retstring[strloc ++] = (obj->usage.sniper ? 'S':'s');
  retstring[strloc ++] = (obj->usage.none ? 'O':'o');
  retstring[strloc++] = '\0';
  
}

void get_obj_bonus(char *retstring,object *obj)
{
  int k;
  retstring[0] = '\0';
  
  for ( k = 0; k < 4 ; k++)
    {
      switch (obj->data.arcomp->bonuses[k].type)
	{
	case (nobonus):
	  strcat (retstring,"N");
	  break;
	case (str):
	  strcat (retstring,"S");
	  break;
	case (sma):
	  strcat (retstring,"I");
	  break;
	case (wis):
	  strcat (retstring,"W");
	  break;
	case (con):
	  strcat (retstring,"C");
	  break;
	case (dex):
	  strcat (retstring,"D");
	  break;
	case (luck):
	  strcat (retstring,"U");
	  break;
	case (looks):
	  strcat (retstring,"L");
	  break;
	case (cha):
	  strcat (retstring,"H");
	  break;
	case (speed):
	  strcat (retstring,"P");
	  break;
	default:
	  strcat (retstring,"?");
	  break;
	}
      sprintf (retstring,"%s %d ",retstring,obj->data.arcomp->bonuses[k].
	       amount);
    }
}


