/*********************************************************/
/* file: player.c                                        */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 may 93                                   */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  None as yet. if you add place your name here or below*/
/*                                                       */
/* description:                                          */
/*   player.c contains subroutines necessary for the     */
/* the player objects in the game                        */
/*                                                       */
/* LICENSING:                                            */
/*   a full licenses is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, named LICENSE             */
/*********************************************************/
#include <player.h>

/***************************************************/
/* move_player                                     */
/*	moves a player,performs special functions  */
/* that relate to a player                         */
/***************************************************/
int
move_player(playerstruct * pl, int dir)
{
  object *             tobj;
  /* exit if player has used up speed */
  if (PL_HAS_MOVED(pl))
    return 0;
  
  if (!move_object(pl->obj, dir) && PL_CHARGES(pl)&&
      OB_DIR_IN_MAP(pl->obj,dir))
    {
      /* only attack if move failed in forward or      */
      /* forward diagonal directions relative to player */
      if (AN_OB_ITEM(pl->obj).held != NULL &&
	  (((abs(dir + 8 - pl->obj->facing)) % 8) <= 1))
	{
	  tobj = AN_OB_ITEM(pl->obj).held;
	  /* only attack if it is a melee weapon */
	  if (IS_WEAPON(tobj) && tobj->subtype == 0)
	    SENDRQ(tobj,use)(tobj, pl->obj, MAP_ADJ_OBJ(pl->obj, dir), dir);
	  return 0;
      } else
	attack_object(MAP_ADJ_OBJ(pl->obj, dir), pl->obj, NULL);
  } else
    if (pl->itemmode == 0)
      {
	pl->top_held = pl->obj->below;
	SET_PL_ITEM_CHANGED(pl);
      }
	SET_PL_HAS_MOVED(pl);
  return 1;
}


/******************************************/
/* shift_items_up                         */
/*   will shift the given players view of */
/* his/her items one item 'up' in every   */
/*  column                                */
/******************************************/
void
shift_items_up(playerstruct * pl)
{
  if (pl->top_held != NULL && pl->top_held->below != NULL)
    {
      pl->top_held = pl->top_held->below;
      SET_PL_ITEM_CHANGED(pl);
    }
}

/******************************************/
/* shift_items_down												*/
/* 		will shift the given players view of*/
/*	his/her items one item 'down' in every*/
/*  column                                */
/******************************************/
void
shift_items_down(playerstruct * pl)
{
  if (pl->top_held != NULL && pl->top_held->above != NULL)
    {
      pl->top_held = pl->top_held->above;
      SET_PL_ITEM_CHANGED(pl);
    }
}

/******************************************/
/* shift_items_left                       */
/*   will shift the given players view of */
/* items one column to the left           */
/******************************************/
void
shift_items_left(playerstruct * pl)
{
  object         *tobj;
  int             tmp;
  for (tmp = ITEM_H - pl->th - 66, tobj = pl->top_held;
       tobj != NULL && tmp > 0; tmp -= 32, tobj = tobj->below);
  if (tobj != NULL)
    {
      pl->top_held = tobj;
      SET_PL_ITEM_CHANGED(pl);
    }
}

/******************************************/
/* shift_items_right                      */
/*   will shift the given players view of */
/* items one column to the right          */
/******************************************/
void
shift_items_right(playerstruct * pl)
{
  object         *tobj;
  int             tmp;
  for (tmp = ITEM_H - pl->th - 66, tobj = pl->top_held;
       tobj != NULL && tmp > 0 && tobj->above != NULL;
       tmp -= 32, tobj = tobj->above);
  if (tobj != NULL)
    {
      pl->top_held = tobj;
      SET_PL_ITEM_CHANGED(pl);
    }
}


/**********************************************/
/* get_selected_item                          */
/*    given the x and y coordinate in the     */
/* item window, get_selected_item will return */
/* the item at that location                  */
/**********************************************/
object         *
get_selected_item(playerstruct * pl, int x, int y)
{
  object         *tobj;
  int             count,
  num_vert;
  
  num_vert = (ITEM_H - pl->th -2)/((ITEM_H - pl->th -2)/64);
  count = (ITEM_H-pl->th-2)/ITEM_ADVANCE;
  count = (x/(ITEM_W/4)*count) +((y-pl->th-2)/num_vert);
  for (tobj = pl->top_held; count > 0 && tobj != NULL;
       count --, tobj = tobj->below);
  return tobj;
}
